@@ -52,11 +52,12 @@ import pandas as pd
 import matplotlib.pyplot as plt
 from scipy.stats import chi2, norm
 from scipy import integrate
+from sklearn.metrics import roc_auc_score
 from statsmodels.nonparametric.smoothers_lowess import lowess
 from IPython.display import display
 
 from .calbelt import CalibrationBelt
-from .metrics import brier, auroc
+from .metrics import brier
 from ._result_types import *
 
 
@@ -128,7 +129,7 @@ class _BaseCalibrationEvaluator:
             self.__devel = DEVEL.INTERNAL
         
         # Define calibration metrics
-        self.__auroc = auroc(self.__y, self.__p)            # Area under the receiver operating curve
+        self.__auroc = roc_auc_score(self.__y, self.__p)    # Area under the receiver operating curve
         self.__brier = brier(self.__y, self.__p,)           # Brier score
         self.__ace = None                                   # Adative calibration error
         self.__mce = None                                   # Maximum calibration error
@@ -225,7 +226,7 @@ class _BaseCalibrationEvaluator:
         if ( (p < 0.0 ).any() or (p > 1.0).any() ):
             raise ValueError("Predicted probabilities y_pred must be in range [0.0 1.0]!")
         if (abs( p.sum() - y.sum() ) < 1e-04 ) and outsample == True:
-            warnings.warn("Please set parameter outsample to 'false' if the evaluated model was fit on this dataset!", "UserWarning")
+            warnings.warn(Warning("Please set parameter outsample to 'false' if the evaluated model was fit on this dataset!"), "UserWarning")
         if ( y.sum() <= 1 ) or ( y.sum() >= (len(y) - 1) ):
             raise ValueError("The number of events/non events in observations can not be less than 1.")
 
@@ -884,4 +885,4 @@ class _BaseCalibrationEvaluator:
 
         ax1.legend(loc='best', bbox_to_anchor=(0.5, 0., 0.5, 0.5))
 
-        return fig
\ No newline at end of file
+        return fig
